
import java.awt.Image;
import javax.swing.ImageIcon;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;

import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Timer;

public class Board extends JPanel implements ActionListener 
{
    private Timer timer;
    private Hero hero;
    private static NetworkClient nc;
    ArrayList<HeroStats> collection;
    private ArrayList<Rectangle> collisionList = new ArrayList();
    private ArrayList<Shot> shotList;

    public Board() 
    {

        addKeyListener(new TAdapter());
        setFocusable(true);
        setBackground(Color.WHITE);
        setDoubleBuffered(true);
        hero = new Hero();
        nc = new NetworkClient();

        loadCollisionList();

        timer = new Timer(5, this);
        timer.start();
    }

    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D)g;

        g2d.drawImage(hero.getImage(), hero.getX(), hero.getY(), this);
        collection = nc.getCollection();     
        for(HeroStats tmp : collection)
        {
            if(tmp.id != nc.id)
                g2d.drawImage(hero.getImage(tmp), (int)tmp.x, (int)tmp.y, this);
        }

        //obrys
        g.setColor(Color.BLUE);
        g.fillRect(0, 0, 1008, 5);
        g.fillRect(0, 0, 5, 740); 
        g.fillRect(0, 710, 1008, 5);
        g.fillRect(1008, 0, 5, 740); 

        //patra
        g.fillRect(100, 650, 250, 5);
        g.fillRect(200, 590, 400, 5);
        
        shotList = nc.getShotList();
        for(Shot tmp2 : shotList)
            g.fillRect(tmp2.x, tmp2.y, 5, 5);
        
        /*Rectangle heroRectangle = hero.getBounds();
        g.fillRect((int)(heroRectangle.getX()), (int)(heroRectangle.getY()), (int)(heroRectangle.getWidth()), (int)(heroRectangle.getHeight()));*/

        Toolkit.getDefaultToolkit().sync();
        g.dispose();
    }

    public void actionPerformed(ActionEvent e) {
        hero.doKeyAction();
        hero.action();    
        hero.sendStats(nc);

        checkCollisions();
        repaint();  
    }

    public void loadCollisionList()
    {
        // obrys
        collisionList.add(new Rectangle(0, 0, 1008, 5));
        collisionList.add(new Rectangle(0, 0, 5, 740));
        collisionList.add(new Rectangle(0, 710, 1008, 5));
        collisionList.add(new Rectangle(1008, 0, 5, 740));
        
        //patra
        collisionList.add(new Rectangle(100, 650, 250, 5));
        collisionList.add(new Rectangle(200, 590, 400, 5));

    }

    public void checkCollisions() 
    {      
        boolean standingSolid = false;
        Rectangle heroRectangle = hero.getBounds();
        for(Rectangle tmp : collisionList)
        {
            // kolize Y - patra, schody, vertikalni predmety
            /*
             * reseni padu na patro -> mene nez 4px, je kvuli fyzice... muze se stat, ze panacek bude mit nohy kvuli akceleraci vrazene hloubeji v zemi, coz by znamenalo propad patrem
             */
            if((heroRectangle.getY()+heroRectangle.getHeight()) >= tmp.getY())
            {
                if((heroRectangle.getY()+heroRectangle.getHeight()) - tmp.getY() < 4)
                    if(tmp.intersects(heroRectangle))
                    {
                        standingSolid = true;
                        hero.setIsOnGround(true); 
                    }
            } 
            /*
             * ochrana proti proskoceni patra
             */
            if(heroRectangle.getY() >= tmp.getY())
            {
                if((heroRectangle.getY() - tmp.getY()) < 5)
                    if(!(((heroRectangle.getX()+heroRectangle.getWidth()/2) < tmp.getX()) || ((heroRectangle.getX()+heroRectangle.getWidth()/2) > (tmp.getX()+tmp.getWidth()))))
                    {
                        hero.setJumpDefaults(); // reseni - problemy se skokem po narazeni hlavy
                        hero.setJump(false); //prerusi skok
                    }
            }
            
            // kolize steny
            /*
             * protoze pouzivam intersection, tak se za stenu povazuje i hrana objektu, coz nechci. Odebiram pixel zespoda a zeshora, coz by vadilo pouze pri 2px tlustem patre, coz nemam
             */
            Rectangle wall = new Rectangle(heroRectangle);
            wall.y -= 1;
            wall.height -= 2;
            if(tmp.intersects(wall))
                if((heroRectangle.getX()) <= tmp.getX())
                    hero.x--;
                else if((heroRectangle.getX()+heroRectangle.getWidth()) >= (tmp.getX()+tmp.getWidth()))
                    hero.x++;
            
        }
        if(!standingSolid)
            hero.setIsOnGround(false);   
              
        

        /*for(Rectangle tmp : collisionList)
        {
        if((heroRectangle.getY()+heroRectangle.getHeight()) >= tmp.getY())
        {
        System.out.println((heroRectangle.getY()+heroRectangle.getHeight())+"  "+ tmp.getY());
        if(tmp.intersects(heroRectangle))
        {
        standingSolid = true;
        hero.setIsOnGround(true); 
        hero.setJump(false); 
        }
        } 
        }
        if(!standingSolid)
        hero.setIsOnGround(false); 

        /*  Rectangle heroRectangle = hero.getBounds();
         * for(Rectangle tmp : collisionList)
        if (hero.getBounds().intersects(tmp))
        {
        hero.setIsOnGround(true); 
        hero.setJump(false); 
        }*/
    }

    private class TAdapter extends KeyAdapter {
        public void keyReleased(KeyEvent e) 
        {
            hero.keyReleased(e, nc);
        }

        public void keyPressed(KeyEvent e) 
        {
            hero.keyPressed(e, nc);
        }
    }
}

